#pragma once
#include <string>
#include <vector>
#include <chrono>
#include "EngineConfig.h"

struct ProfileNode
{
    std::string name;
    double startMs = 0.0;
    double durationMs = 0.0;
    double visualStartMs = 0.0;
    double visualDurationMs = 0.0;
    std::vector<ProfileNode> children;


    
    ProfileNode() = default;
    

    ProfileNode(const std::string &n, double start)
        : name(n), startMs(start)
    {
        children.reserve(8);
    }
};

class HierarchicalProfiler
{
public:
    void BeginFrame();
    void EndFrame();

    void BeginSection(const std::string &name);
    void EndSection();

    void BeginEngineSection(const std::string &name);
    void EndEngineSection();

    const std::vector<ProfileNode> &GetFrames() const;
    const ProfileNode *GetLatestFrame() const;

private:
    using Clock = std::chrono::high_resolution_clock;
    Clock::time_point startTime;
    std::vector<Clock::time_point> sectionStartTimes;
    std::vector<ProfileNode *> currentStack;
    ProfileNode root{"Frame", 0.0};

    std::vector<ProfileNode> frameHistory;
    static constexpr size_t maxFrames = 30;

    size_t ProfilesLastFrame = 128;
};

extern HierarchicalProfiler profiler;

// RAII Scoped Profiling (Zero-overhead when disabled)
struct ScopedProfile
{
    ScopedProfile(const std::string &name)
    {
        if (g_engineConfig.settings.profile_enabled)
            profiler.BeginSection(name);
    }
    ~ScopedProfile()
    {
        if (g_engineConfig.settings.profile_enabled)
            profiler.EndSection();
    }
};

struct ScopedEngineProfile
{
    ScopedEngineProfile(const std::string &name)
    {
        if (g_engineConfig.settings.profile_enabled && g_engineConfig.settings.profile_editor)
            profiler.BeginSection(name);
    }
    ~ScopedEngineProfile()
    {
        if (g_engineConfig.settings.profile_enabled && g_engineConfig.settings.profile_editor)
            profiler.EndSection();
    }
};

#define CONCAT_IMPL(a, b) a##b
#define CONCAT(a, b) CONCAT_IMPL(a, b)

#define PROFILE_SCOPE(label) \
    ScopedProfile CONCAT(_scopedProfile_, __LINE__)(label)

#define PROFILE_ENGINE_SCOPE(label) \
    ScopedEngineProfile CONCAT(_scopedEngineProfile_, __LINE__)(label)
